use std::str::Lines;
use std::time::Instant;

pub fn main() {
    let input = include_str!("../../../input/day01.txt");

    let mut start = Instant::now();
    let res1 = part01(input.lines());
    println!("part1: {} ({} µs)", res1, start.elapsed().as_micros());

    start = Instant::now();
    let res2 = part02(input.lines());
    println!("part2: {} ({} µs)", res2, start.elapsed().as_micros());
}

fn part01(lines: Lines) -> u64 {
    let mut acc: u64 = 0;
    for line in lines {
        let mut first: u8 = 10;
        let mut last: u8 = 10;

        for c in line.chars() {
            if !(c.is_numeric()) {
                continue;
            }

            let digit = c.to_digit(10).unwrap() as u8;

            if first == 10 {
                first = digit;
                last = digit;
            } else {
                last = digit;
            }
        }

        acc += (10 * first + last) as u64;
    }

    acc
}

fn part02(lines: Lines) -> u32 {
    lines.map(|line| parse_line(line)).sum()
}

const NUM_CANDIDATES: [&str; 9] = [
    "one", "two", "three", "four", "five", "six", "seven", "eight", "nine",
];

fn parse_line(line: &str) -> u32 {
    let digits: Vec<u32> = (0..line.len())
        .into_iter()
        .map(|i| &line[i..])
        .map(|sub| {
            if let Some(digit) = sub
                .chars()
                .next()
                .filter(|c| c.is_digit(10))
                .map(|c| c.to_digit(10).unwrap())
            {
                Some(digit)
            } else {
                NUM_CANDIDATES
                    .iter()
                    .filter(|&&num_str| sub.starts_with(num_str))
                    .map(|&num_str| NUM_CANDIDATES.into_iter().position(|c| c.eq(num_str)))
                    .filter(|opt_pos| opt_pos.is_some())
                    .map(|opt_pos| (opt_pos.unwrap() + 1) as u32)
                    .next()
            }
        })
        .filter(|o| o.is_some())
        .map(|o| o.unwrap())
        .collect();

    digits.get(0).unwrap() * 10 + digits.get(digits.len() - 1).unwrap()
}

#[test]
fn parse_line_tests() {
    let test = |n: u32, s| assert_eq!(n, parse_line(s));

    // weird boundaries
    test(11, "tone");
    test(88, "threight");

    // simple cases
    test(22, "2");
    test(22, "two");

    // multiple simple
    test(23, "two3");
    test(14, "1two3asdf4");
    test(18, "1two3asdfeight");

    // examples from aoc
    test(29, "two1nine");
    test(83, "eightwothree");
    test(13, "abcone2threexyz");
    test(24, "xtwone3four");
    test(42, "4nineeightseven2");
    test(14, "zoneight234");
    test(76, "7pqrstsixteen");

    // reddit
    test(18, "oneightwoneight");

    // another shit
    test(91, "9twone");
    test(21, "twone");
}
