use std::str::Lines;
use std::time::Instant;

#[derive(Debug)]
struct Game {
    id: u32,
    shown: Vec<ShownCubes>,
}

#[derive(Debug)]
struct ShownCubes {
    red: u32,
    green: u32,
    blue: u32,
}

fn main() {
    let input = include_str!("../../../input/day02.txt");
    let mut start = Instant::now();
    let games = parse_input(input.lines());
    println!("Parsing took {} µs", start.elapsed().as_micros());

    start = Instant::now();
    let res1 = games
        .iter()
        .filter(|g| {
            g.shown
                .iter()
                .all(|shown| shown.red <= 12 && shown.green <= 13 && shown.blue <= 14)
        })
        .map(|g| g.id)
        .sum::<u32>();
    println!("part1: {} ({} µs)", res1, start.elapsed().as_micros());

    start = Instant::now();
    let res2 = games
        .iter()
        .map(|g| {
            let red_max = g.shown.iter().map(|s| s.red).max().unwrap_or(0);
            let green_max = g.shown.iter().map(|s| s.green).max().unwrap_or(0);
            let blue_max = g.shown.iter().map(|s| s.blue).max().unwrap_or(0);

            red_max * green_max * blue_max
        })
        .sum::<u32>();
    println!("part2: {} ({} µs)", res2, start.elapsed().as_micros());
}

fn parse_input(lines: Lines) -> Vec<Game> {
    let mut games = vec![];

    for line in lines {
        let mut game_shows = line.split(":");
        let game_id = game_shows
            .next()
            .unwrap()
            .split(' ')
            .nth(1)
            .unwrap()
            .parse::<u32>()
            .unwrap();

        let mut shown = vec![];
        for shown_str in game_shows.next().unwrap().split(';') {
            let (mut red, mut green, mut blue) = (0u32, 0u32, 0u32);

            for shown_cube_str in shown_str.split(',').into_iter() {
                let mut count_color = shown_cube_str.trim().split(' ');
                let count = count_color.next().unwrap().parse::<u32>().unwrap();
                let color = count_color.next().unwrap();

                match color {
                    "red" => red = count,
                    "green" => green = count,
                    "blue" => blue = count,
                    _ => panic!("unknown color {}", color),
                }
            }

            shown.push(ShownCubes { red, green, blue });
        }

        games.push(Game { id: game_id, shown })
    }

    games
}
