use std::cmp::min;
use std::time::Instant;

type CardWinCounts = Vec<u32>;

fn main() {
    let input = include_str!("../../../input/day04.txt");
    let mut start = Instant::now();
    let cards = read_win_counts(input);
    println!(
        "Parsing + preprocessing took {} µs",
        start.elapsed().as_micros()
    );

    start = Instant::now();
    let res1 = calculate_points_part1(&cards);
    println!("part1: {} ({} µs)", res1, start.elapsed().as_micros());

    start = Instant::now();
    let res2 = calculate_total_cards_part2(&cards);
    println!("part2: {} ({} µs)", res2, start.elapsed().as_micros());
}

fn read_win_counts(input: &str) -> CardWinCounts {
    input
        .lines()
        .into_iter()
        .map(|line| get_card_numbers(line))
        .map(|(winning, having)| having.iter().filter(|n| winning.contains(n)).count() as u32)
        .collect()
}

fn get_card_numbers(line: &str) -> (Vec<u32>, Vec<u32>) {
    let mut numbers_it = line.split(':').nth(1).unwrap().split('|');

    (
        get_whitespace_separated_numbers(numbers_it.next().unwrap()),
        get_whitespace_separated_numbers(numbers_it.next().unwrap()),
    )
}

fn calculate_points_part1(cards: &CardWinCounts) -> u32 {
    cards
        .iter()
        .map(|&win_count| match win_count {
            0 => 0,
            n => 1 << (n - 1),
        })
        .sum()
}

fn get_whitespace_separated_numbers(input: &str) -> Vec<u32> {
    input
        .split(char::is_whitespace)
        .into_iter()
        .filter_map(|n_str| n_str.trim().parse::<u32>().ok())
        .collect()
}

fn calculate_total_cards_part2(cards: &CardWinCounts) -> u32 {
    let mut count = vec![1; cards.len()];

    for (idx, &win_count) in cards.iter().enumerate() {
        let this_count = count[idx];

        for i in (idx + 1)..min(idx + 1 + win_count as usize, cards.len()) {
            count[i] += this_count;
        }
    }

    count.iter().sum()
}

#[test]
fn check_example() {
    let input = "Card 1: 41 48 83 86 17 | 83 86  6 31 17  9 48 53
Card 2: 13 32 20 16 61 | 61 30 68 82 17 32 24 19
Card 3:  1 21 53 59 44 | 69 82 63 72 16 21 14  1
Card 4: 41 92 73 84 69 | 59 84 76 51 58  5 54 83
Card 5: 87 83 26 28 32 | 88 30 70 12 93 22 82 36
Card 6: 31 18 13 56 72 | 74 77 10 23 35 67 36 11";

    let cards = read_win_counts(String::from(input));
    assert_eq!(13, calculate_points_part1(&cards));
    assert_eq!(30, calculate_total_cards_part2(&cards));
}
