use rayon::prelude::*;
use std::time::Instant;

fn main() {
    let input = include_str!("../../../input/day09.txt");
    let mut start = Instant::now();
    let parsed = parse(input);
    println!(
        "Parsing + preprocessing took {} µs",
        start.elapsed().as_micros()
    );

    start = Instant::now();
    let res1 = part1(&parsed);
    println!("part1: {} ({} µs)", res1, start.elapsed().as_micros());

    start = Instant::now();
    let res2 = part2(&parsed);
    println!("part2: {} ({} µs)", res2, start.elapsed().as_micros());
}

fn part1(seqs: &Vec<Vec<Vec<i64>>>) -> i64 {
    let mut acc = 0;
    for seq in seqs {
        let mut prev = 0;

        for std in seq.iter().rev().skip(1) {
            prev = std.last().unwrap() + prev;
        }

        acc += prev;
    }

    acc
}

fn part2(seqs: &Vec<Vec<Vec<i64>>>) -> i64 {
    let mut acc = 0;
    for seq in seqs {
        let mut prev = 0;

        for std in seq.iter().rev().skip(1) {
            prev = std.first().unwrap() - prev;
        }

        acc += prev;
    }

    acc
}

fn parse(input: &str) -> Vec<Vec<Vec<i64>>> {
    input
        .par_lines()
        .map(|line| numbers(line))
        .map(|nums| sequences(nums))
        .collect()
}

fn sequences(numbers: Vec<i64>) -> Vec<Vec<i64>> {
    let mut vecs = vec![];
    let mut last = numbers;

    while !last.iter().all(|n| *n == 0) {
        let next: Vec<i64> = last.windows(2).map(|ns| ns[1] - ns[0]).collect();
        vecs.push(last);
        last = next;
    }
    vecs.push(last);

    vecs
}

fn numbers(line: &str) -> Vec<i64> {
    line.split(char::is_whitespace)
        .map(|ns| ns.parse::<i64>().unwrap())
        .collect()
}
