#[derive(Debug, Eq, PartialEq)]
enum SpringStatus {
    Good,
    Broken,
    Unknown,
}

impl From<char> for SpringStatus {
    fn from(value: char) -> Self {
        match value {
            '.' => SpringStatus::Good,
            '#' => SpringStatus::Broken,
            '?' => SpringStatus::Unknown,
            _ => panic!("Unknown spring status"),
        }
    }
}

#[derive(Debug, Eq, PartialEq)]
struct ConditionReport {
    status: Vec<SpringStatus>,
    broken_counts: Vec<u32>,
}

fn main() {
    let input = include_str!("../../../input/day12.txt");
    let parsed = parse(input);

    let res1 = part1(&parsed);
}

fn parse(input: &str) -> Vec<ConditionReport> {
    input
        .lines()
        .map(|line| {
            let mut line_it = line.split(' ');
            (line_it.next().unwrap(), line_it.next().unwrap())
        })
        .map(|(status, broken)| ConditionReport {
            status: status.chars().map(SpringStatus::from).collect(),
            broken_counts: broken
                .split(',')
                .filter_map(|n_str| n_str.trim().parse::<u32>().ok())
                .collect(),
        })
        .collect()
}

fn part1(reports: &Vec<ConditionReport>) -> u64 {
    0
}

fn fix_report(report: &ConditionReport) -> ConditionReport {
    ConditionReport {
        status: vec![],
        broken_counts: vec![],
    }
}

#[cfg(test)]
mod test {
    use crate::{fix_report, ConditionReport, SpringStatus};

    #[test]
    fn fixes_reports() {
        assert_eq!(
            ConditionReport {
                status: vec![SpringStatus::Broken],
                broken_counts: vec![]
            },
            fix_report(&ConditionReport {
                status: vec![SpringStatus::Unknown],
                broken_counts: vec![1]
            })
        );

        assert_eq!(
            ConditionReport {
                status: vec![SpringStatus::Good, SpringStatus::Broken, SpringStatus::Good],
                broken_counts: vec![]
            },
            fix_report(&ConditionReport {
                status: vec![
                    SpringStatus::Unknown,
                    SpringStatus::Broken,
                    SpringStatus::Unknown
                ],
                broken_counts: vec![1]
            })
        );
    }
}
